@@ -167,7 +167,8 @@ static int openpty (int *amaster, int *aslave, char *name)
 
 static int forkpty (int *amaster, char *name,...)
 {
-    int master, slave, pid;
+    int master, slave;
+    pid_t pid;
     if (openpty (&master, &slave, name) == -1)
 	return -1;
     switch (pid = fork ()) {
@@ -175,15 +176,29 @@ static int forkpty (int *amaster, char *name,...)
 	return -1;
     case 0:
 	close (master);
-	login_tty (slave);
-	setsid ();
-	/* Set tty as session leader, steal it if invoked as su */
-	if (ioctl (slave, TIOCSCTTY, (char *) 1) == -1)
-	    return (-1);
-	dup2 (fd, 0);
-	dup2 (fd, 1);
-	dup2 (fd, 2);
-	if (fd > 2)
+#ifdef HAVE_SETSID
+	pid = setsid ();
+#define HAVE_PID_THIS_TTY
+#elif defined (HAVE_SETPGRP)
+	pid = setpgrp (0, 0);
+#define HAVE_PID_THIS_TTY
+#endif
+#ifdef TIOCSCTTY
+	ioctl (slave, TIOCSCTTY, 0);
+#ifdef HAVE_PID_THIS_TTY
+#elif defined (HAVE_TCSETPGRP)
+	tcsetpgrp (slave, pid);
+#elif defined (TIOCSPGRP)
+	ioctl (slave, TIOCSPGRP, &pid);
+#endif
+#endif
+	close (0);
+	dup (slave);
+	close (1);
+	dup (slave);
+	close (2);
+	dup (slave);
+	if (slave > 2)
 	    close (slave);
 	return 0;
     }
@@ -273,7 +288,9 @@ static pid_t open_under_pty (int *in, int *out, char **line, const char *file, c
 	return -1;
     if (p) {
 	int not = 0;
+#if 0
 	ioctl (master, FIONBIO, &not);
+#endif
 	*in = master;
 	*out = master;
 	*line = (char *) strdup (l);
